//Copyright 2005 - 2010 The MathWorks, Inc.
using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Runtime.InteropServices;

namespace Gene_Expression_C_Sharp
{
	/// <summary>
	/// Summary description for ScatterPlot.
	/// </summary>
	public class GeneratePlot : System.Web.UI.Page
	{
		private void Page_Load(object sender, System.EventArgs e)
		{
			Object imageData = null;

			try
			{
				geneAnalysis.IgeneAnalysisclass analyzer = (geneAnalysis.IgeneAnalysisclass)Session.Contents["analyzer"];
				String plotType = Context.Request["plot"];

				if(plotType == "statistics")
				{
					analyzer.getStatisticsPlot(1, ref imageData);
				}
				else if(plotType == "profile")
				{
					analyzer.getProfilePlot(1, ref imageData);
				}
				else if(plotType == "scatter")
				{
					analyzer.getScatterPlot(1, ref imageData);
				}
				else if(plotType == "lastOperation")
				{
					//Pull the plot from the session context.  This will be shoved in as a result of
					//the last analysis operation on the clustering page
					imageData = Session.Contents["lastOperationImage"];
				}
			}
			catch(COMException ex)
			{
				//Rethrow to the calling class
				throw ex;
			}

			if(imageData != null)
			{
				// Put user code to initialize the page here
				byte[,,] af = (byte[,,])imageData;
			
				
				//Get a bitmap object from the raw image data
				System.Drawing.Image m = generateBitMap(af);

				// resize or create a thumbnail as required
				if (Context.Request["thumbnail"] != null)
				{
					// Need to load the image, resize it, and stream to the client.
					// Calculate the scale so as not to stretch or distort the image.
					float scale = 250.0f / System.Math.Max(m.Height, m.Width);
					System.Drawing.Image thumb = m.GetThumbnailImage(  (int)(m.Width * scale), 
						(int)(m.Height * scale), 
						null, 
						System.IntPtr.Zero);

					m = thumb;
				}
				else
				{
					//Resize the image if there is size information available
					if(this.Session.Contents["height"] != null && this.Session.Contents["width"] != null)
					{
						double height = ((Unit)this.Session.Contents["height"]).Value;
						double width = ((Unit)this.Session.Contents["width"]).Value;
						m = new Bitmap(m, (Int32)height, (Int32)width);
					}
				}

				//Write the data to the response stream
				Response.ContentType = "image/jpeg";
				m.Save(this.Response.OutputStream, ImageFormat.Jpeg);		
				m.Dispose();
			}
			else
			{
				//If no image was generated, throw up our default image
				Response.ContentType = "image/jpeg";
				System.Drawing.Image m = System.Drawing.Image.FromFile(AppDomain.CurrentDomain.BaseDirectory + "images/microarray.bmp");
				if(this.Session.Contents["height"] != null && this.Session.Contents["width"] != null)
				{
					double height = ((Unit)this.Session.Contents["height"]).Value;
					double width = ((Unit)this.Session.Contents["width"]).Value;
					m = new Bitmap(m, (Int32)height, (Int32)width);
				}

				m.Save(this.Response.OutputStream, ImageFormat.Jpeg);
				m.Dispose();
			}
		}

		public static Bitmap generateBitMap(byte[,,] imageData)
		{
			Bitmap m = new Bitmap(imageData.GetLength(1), imageData.GetLength(0));
			//Build up the bitmap from the MATLAB intesity data
			//MATLAB is using the default RGB color map with 
			//an alpha channel of 1.0 (255), so just shove the
			//data straight in

			BitmapData bmd = m.LockBits(new Rectangle(0, 0, imageData.GetLength(1), imageData.GetLength(0)), 
				System.Drawing.Imaging.ImageLockMode.ReadWrite, 
				m.PixelFormat);

			int pixelSize = 4;
			for(int idx = 0; idx < imageData.GetLength(0); idx++)
			{
				for(int jdx = 0; jdx < imageData.GetLength(1); jdx++)
				{
					//swap x,y to account for MATLAB column/row orientation
					Marshal.WriteByte(bmd.Scan0, (bmd.Stride * idx) + (pixelSize * jdx) , imageData[idx + 1,jdx + 1,3]);
					Marshal.WriteByte(bmd.Scan0, (bmd.Stride * idx) + (pixelSize * jdx + 1) , imageData[idx + 1,jdx + 1,2]);
					Marshal.WriteByte(bmd.Scan0, (bmd.Stride * idx) + (pixelSize * jdx + 2) , imageData[idx + 1,jdx + 1,1]);
					Marshal.WriteByte(bmd.Scan0, (bmd.Stride * idx) + (pixelSize * jdx + 3) , 255);
				}
			}

			m.UnlockBits(bmd);

			return m;
		}
		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
